use colored::Colorize;
use serde::{Deserialize, Serialize};
use std::io::Write;

#[derive(Debug, Deserialize, Serialize)]
pub struct JiraIssue {
    pub key: String,
    #[serde(rename = "self")]
    pub href: String,
    pub fields: JiraIssueResponseFields,
}

#[derive(Debug, Deserialize, Serialize)]
pub struct JiraIssueResponseFields {
    pub summary: String,
    pub status: Status,
    pub created: chrono::DateTime<chrono::Utc>,
    pub priority: Priority,
    pub assignee: Person,
    pub reporter: Person,
    pub creator: Person,
    #[serde(rename = "duedate")]
    pub due_date: Option<chrono::NaiveDate>,
}

#[derive(Debug, Deserialize, Serialize)]
pub struct Status {
    pub name: String,
}

#[derive(Debug, Deserialize, Serialize)]
pub struct Priority {
    pub name: String,
    pub id: String,
}

#[derive(Debug, Deserialize, Serialize)]
pub struct Person {
    #[serde(rename = "self")]
    pub href: String,
    #[serde(rename = "displayName")]
    pub display_name: String,
    #[serde(rename = "accountId")]
    pub account_id: String,
    #[serde(rename = "emailAddress")]
    pub email_address: String,
}

pub fn display_issues_pretty(issues: &[JiraIssue]) -> Result<(), Box<dyn std::error::Error>> {
    println!("Found {} issues:", issues.len());
    println!("{:-<80}", "");

    for issue in issues {
        let mut tw = tabwriter::TabWriter::new(vec![]);
        writeln!(tw, "{}:\t{}", "Key".blue(), issue.key)?;
        writeln!(tw, "{}:\t{}", "Summary".blue(), issue.fields.summary)?;
        writeln!(tw, "{}:\t{}", "Status".blue(), issue.fields.status.name)?;
        writeln!(tw, "{}:\t{}", "Created".blue(), issue.fields.created)?;
        writeln!(tw, "{}:\t{:?}", "Due Date".blue(), issue.fields.due_date)?;
        writeln!(tw, "{}:\t{}", "URL".blue(), issue.href.underline())?;
        tw.flush().unwrap();

        let written = String::from_utf8(tw.into_inner().unwrap()).unwrap();
        print!("{}", written);
        println!("{:-<80}", "");
    }

    Ok(())
}

pub fn display_issues_json(issues: &[JiraIssue]) -> Result<(), Box<dyn std::error::Error>> {
    let j = serde_json::to_string_pretty(issues)?;
    println!("{}", j);
    Ok(())
}
